/*******************************************************************************
 *  pong game   |   version 0.9     |   2-Clause BSD License    |   2014-06-05
 *  James Hendrie                   |   hendrie.james@gmail.com
 ******************************************************************************/
#ifndef UTIL_H
#include "util.h"
#endif

/*
--------------------------------------------------------------------------------
                                      INIT
--------------------------------------------------------------------------------
*/
bool init( void )
{
    /*  Init SDL */
    if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK ) < 0 )
    {
        printf("%s\n", SDL_GetError() );
        return( false );
    }

    /*  Init SDL_mixer */
    if( Mix_OpenAudio( 44100, MIX_DEFAULT_FORMAT, 2, 2048 ) < 0 )
    {
        printf("%s\n", Mix_GetError() );
        return( false );
    }

    /*  Init SDL_ttf */
    if( TTF_Init() == -1 )
    {
        printf("%s\n", TTF_GetError() );
        return( false );
    }

    /*  Create window */
    gWindow = SDL_CreateWindow( "Pong clone",
            SDL_WINDOWPOS_UNDEFINED, SDL_WINDOWPOS_UNDEFINED,
            SCREEN_WIDTH, SCREEN_HEIGHT, SDL_WINDOW_SHOWN );
    if( gWindow == NULL )
    {
        printf("%s\n", SDL_GetError() );
        return( false );
    }

    /*  Create renderer */
    gRenderer = SDL_CreateRenderer( gWindow, -1, SDL_RENDERER_ACCELERATED );
    if( gRenderer == NULL )
    {
        printf("%s\n", SDL_GetError() );
        return( false );
    }


    /*  Joystick stuff */
    if( SDL_NumJoysticks() > 0 )
    {
        /*  If we have at least one joystick plugged in, initialize it */
        pad1 = SDL_JoystickOpen( 0 );
        if( pad1 == NULL )
        {
            printf("Warning:  Unable to init gamepad 1:  %s\n",
                    SDL_GetError() );
        }

        /*  Two?  Initialize both */
        if( SDL_NumJoysticks() > 1 )
        {
            pad2 = SDL_JoystickOpen( 1 );
            if( pad2 == NULL )
            {
                printf("Warning:  Unable to init gamepad 2:  %s\n",
                        SDL_GetError() );
            }
        }
    }

    /*  If we're here, we're good */
    return( true );
}




/*
--------------------------------------------------------------------------------
                                  INIT_PADDLES
--------------------------------------------------------------------------------
*/
void init_paddles( void )
{
    /*  Create the paddle objects */
    for( int i = 0; i < 2; ++i )
    {
        paddles[ i ] = new Paddle();
        paddles[ i ]->set_width( 20 );
        paddles[ i ]->set_height( 100 );

        paddles[ i ]->set_velocity( 0, 0 );
        paddles[ i ]->set_max_velocity( 100 );
    }

    /*  Initialize the paddle position */
    paddles[ 0 ]->set_position( 10,
            ( SCREEN_HEIGHT - paddles[ 0 ]->get_height() ) /2 );
    paddles[ 0 ]->init_collider();      //  Init collision box
    paddles[ 0 ]->set_player( 0 );      //  Player status (player 1 )
    paddles[ 0 ]->set_ai( false );      //  Disable AI on this paddle


    /*  Init paddle position */
    paddles[ 1 ]->set_position( ( SCREEN_WIDTH - paddles[ 1 ]->get_width() -10),
           ( SCREEN_HEIGHT - paddles[ 1 ]->get_height() ) /2 );
    paddles[ 1 ]->init_collider();      //  Init collision box
    paddles[ 1 ]->set_player( 1 );      //  Set this to player 2

    /*  If we have one joystick or fewer plugged in */
    if( SDL_NumJoysticks() < 2 )
    {
        paddles[ 1 ]->set_ai( true );           //  Set this paddle to AI
        paddles[ 1 ]->set_difficulty( 2 );      //  Set difficulty to 2
    }
}


/*
--------------------------------------------------------------------------------
                                   INIT_BALL
--------------------------------------------------------------------------------
*/
void init_ball( void )
{
    ball = new Ball();                  //  Create ball
    ball->set_position( 200, 200 );     //  Init position
    ball->set_width( 16 );              //  Init dimensions
    ball->set_height( 16 );

    ball->set_max_velocity( 12 );       //  Set max velocity to 12

    ball->set_dx( 8 );                  //  Set X direction to positive 8
    ball->set_dy( 8 );                  //  Set Y direction to positive 8

    ball->init_collider();              //  Init collision box
}


/*
--------------------------------------------------------------------------------
                                   INIT_FONTS
--------------------------------------------------------------------------------
*/
bool init_fonts( void )
{
    /*  Default 'bigger' font */
    gFont = TTF_OpenFont( "data/Ubuntu-L.ttf", 64 );
    if( gFont == NULL )
    {
        printf("Could not open font!  TTF Error:  %s\n", TTF_GetError() );
        return( false );
    }

    /*  Smaller font for the help screen */
    gFontHelp = TTF_OpenFont( "data/Ubuntu-L.ttf", 24 );
    if( gFontHelp == NULL )
    {
        printf("Could not open font!  TTF Error:  %s\n", TTF_GetError() );
        return( false );
    }

    return( true );
}


/*
--------------------------------------------------------------------------------
                                 INIT_TEXTURES
--------------------------------------------------------------------------------
*/
void init_textures( void )
{
    /*  Create the textures for the score display */
    for( int i = 0; i < 2; ++i )
    {
        gTextures[ i ] = new Texture();
        gTextures[ i ]->create_texture_from_string( gFont, "00", fgColor );
    }

}


/*
--------------------------------------------------------------------------------
                                  INIT_SCORES
--------------------------------------------------------------------------------
*/
void init_scores( void )
{
    /*  Init the scores */
    for( int i = 0; i < 2; ++i )
    {
        gScores[ i ] = new Score();             //  Create new score object
        gScores[ i ]->set_texture_index( i );   //  Set score texture index
        gScores[ i ]->set_score( 0 );           //  Set the score's value
        gScores[ i ]->set_paddle( i );          //  Which paddle owns this score
    }

    /*  Offset is score's width times four */
    int offSet = gScores[ 1 ]->get_width() * 4;

    /*  Create a temporary score object pointer, assign to gScores[0] */
    Score *s = gScores[ 0 ];
    s->set_position(
            ( ( SCREEN_WIDTH - s->get_width() ) /2 ) - offSet,
            20 );       //  Set position

    /*  Same deal here */
    s = gScores[ 1 ];
    s->set_position(
            ( ( SCREEN_WIDTH - s->get_width() ) /2 ) + offSet,
            20 );

}


/*
--------------------------------------------------------------------------------
                              CONFIG_HELP_TEXTURES
--------------------------------------------------------------------------------
*/
void config_help_textures( void )
{
    /*
     *  Here we create our text color.  If the foreground and background colors
     *  are the same, we change the text color to the outline color.  Otherwise
     *  it's set to the foreground color.
     */
    SDL_Color textColor;
    if( colors_match( bgColor, fgColor ) )
        textColor = outlineColor;
    else
        textColor = fgColor;

    /*
     *  ----    Create all of the keyboard help text textures    ----
     */

    gHelpScreen[ T_HELP_KEYBOARD ]->create_texture_from_string( gFontHelp,
            "Keyboard controls:", textColor );

    gHelpScreen[ T_HELP_GO ]->create_texture_from_string( gFontHelp,
            "SPACE to launch ball", textColor );

    gHelpScreen[ T_HELP_HELP ]->create_texture_from_string( gFontHelp,
            "H to toggle help text", textColor );

    gHelpScreen[ T_HELP_CYCLE_DIFFICULTY ]->
        create_texture_from_string( gFontHelp,
                "D to cycle difficulty", textColor );

    gHelpScreen[ T_HELP_TOGGLE_SOUND ]->create_texture_from_string( gFontHelp,
            "S to toggle sound on/off", textColor );

    gHelpScreen[ T_HELP_TOGGLE_FG ]->create_texture_from_string( gFontHelp,
            "1 to cycle foreground color", textColor );

    gHelpScreen[ T_HELP_TOGGLE_BG ]->create_texture_from_string( gFontHelp,
            "2 to cycle background color", textColor );

    gHelpScreen[ T_HELP_TOGGLE_LINE ]->create_texture_from_string( gFontHelp,
            "3 to toggle middle line on/off", textColor );

    gHelpScreen[ T_HELP_TOGGLE_SCORES ]->create_texture_from_string( gFontHelp,
            "4 to toggle scores on/off", textColor );


    /*  Set position for keyboard help text header */
    gHelpScreen[ T_HELP_KEYBOARD ]->set_position(
            SCREEN_WIDTH/5,
            SCREEN_HEIGHT/20 );

    /*  Set positions for the rest of the keyboard help text */
    for( int i = 1; i < T_HELP_TOTAL; ++i )
    {
        gHelpScreen[ i ]->set_position(
                SCREEN_WIDTH/3.5,
                ( gHelpScreen[ i ]->get_height() * i ) +
            SCREEN_HEIGHT/20 );
    }


    /*
     *  ----    Create all of the gamepad help text textures    ----
     */

    gHelpScreenGamepad[ T_HELP_GAMEPAD ]->create_texture_from_string( gFontHelp,
            "Gamepad controls:", textColor );

    gHelpScreenGamepad[ T_HELP_GO_G ]->create_texture_from_string( gFontHelp,
            "START to launch ball", textColor );

    gHelpScreenGamepad[ T_HELP_HELP_G ]->create_texture_from_string( gFontHelp,
            "SELECT to toggle help text", textColor );

    gHelpScreenGamepad[ T_HELP_CYCLE_DIFFICULTY_G ]->create_texture_from_string(
            gFontHelp, "Y to cycle difficulty", textColor );

    gHelpScreenGamepad[ T_HELP_TOGGLE_SOUND_G ]->create_texture_from_string(
            gFontHelp, "X to toggle sound on/off", textColor );

    gHelpScreenGamepad[ T_HELP_TOGGLE_FG_G ]->create_texture_from_string(
            gFontHelp, "A to cycle foreground color", textColor );

    gHelpScreenGamepad[ T_HELP_TOGGLE_BG_G ]->create_texture_from_string(
            gFontHelp, "B to cycle background color", textColor );

    gHelpScreenGamepad[ T_HELP_TOGGLE_LINE_G ]->create_texture_from_string(
            gFontHelp, "Left bumper to toggle middle line on/off", textColor );

    gHelpScreenGamepad[ T_HELP_TOGGLE_SCORES_G ]->create_texture_from_string(
            gFontHelp, "Right bumper to toggle scores on/off",textColor);


    /*  Set position for gamepad help text header */
    gHelpScreenGamepad[ T_HELP_GAMEPAD ]->set_position(
            SCREEN_WIDTH/5,
            SCREEN_HEIGHT/2 );

    /*  Set positions for the rest of the gamepad help text textures */
    for( int i = 1; i < T_HELP_TOTAL_G; ++i )
    {
        gHelpScreenGamepad[ i ]->set_position(
                SCREEN_WIDTH/3.5,
                ( gHelpScreenGamepad[ i ]->get_height() * i ) +
                SCREEN_HEIGHT/2 );
    }
}



/*
--------------------------------------------------------------------------------
                           CONFIG_GAME_OVER_TEXTURES
--------------------------------------------------------------------------------
*/
void config_game_over_textures( void )
{
    /*
     *  Reset the game over screen.  The parameter is which paddle has won
     *  (0 = first player, 1 = second player).  Default (init) is 0.
     */
    reset_game_over_screen( 0 );

    /*  Set positions for all of the game over text textures */
    for( int i = 0; i < T_GAME_OVER_TOTAL; ++i )
    {
        gGameOverTextures[ i ]->set_position(
                ( SCREEN_WIDTH - gGameOverTextures[ i ]->get_width() ) / 2,
                ( gGameOverTextures[ i ]->get_height() * i ) + 100 );
    }
}



/*
--------------------------------------------------------------------------------
                               INIT_HELP_TEXTURES
--------------------------------------------------------------------------------
*/
void init_help_textures( void )
{
    /*  Assign memory for the textures for the keyboard help text */
    for( int i = 0; i < T_HELP_TOTAL; ++i )
    {
        gHelpScreen[ i ] = new Texture();

    }

    /*  Assign memory for the textures for the gamepad help text */
    for( int i = 0; i < T_HELP_TOTAL_G; ++i )
    {
        gHelpScreenGamepad[ i ] = new Texture();
    }

    /*  Configure all the help text textures */
    config_help_textures();
}



/*
--------------------------------------------------------------------------------
                            INIT_GAME_OVER_TEXTURES
--------------------------------------------------------------------------------
*/
void init_game_over_textures( void )
{
    /*  Create the game over text texture objects */
    for( int i = 0; i < T_GAME_OVER_TOTAL; ++i )
    {
        gGameOverTextures[ i ] = new Texture();
    }

    /*  Configure the textures */
    config_game_over_textures();
}



/*
--------------------------------------------------------------------------------
                                     CLOSE
--------------------------------------------------------------------------------
*/
void close( void )
{
    /*  Destroy all normal global textures */
    for( int i = 0; i < T_TOTAL; ++i )
    {
        delete gTextures[i];
        delete gScores[ i ];
    }

    /*  Destroy help screen textures for keyboard */
    for( int i = 0; i < T_HELP_TOTAL; ++i )
    {
        delete gHelpScreen[ i ];
    }

    /*  Destroy help screen textures for gamepad */
    for( int i = 0; i < T_HELP_TOTAL_G; ++i )
    {
        delete gHelpScreenGamepad[ i ];
    }

    /*  Destroy textures for game over screen */
    for( int i = 0; i < T_GAME_OVER_TOTAL; ++i )
    {
        delete gGameOverTextures[ i ];
    }

    /*  Delete all the colors */
    for( int i = 0; i < TOTAL_COLORS; ++i )
    {
        delete colors[ i ];
    }

    /*  Destroy renderer and window */
    SDL_DestroyRenderer( gRenderer );
    SDL_DestroyWindow( gWindow );
    gRenderer = NULL;
    gWindow = NULL;

    /*  Close game controllers */
    if( pad1 != NULL )
    {
        SDL_JoystickClose( pad1 );
        pad1 = NULL;
    }

    if( pad2 != NULL )
    {
        SDL_JoystickClose( pad2 );
        pad2 = NULL;
    }

    /*  Free the sounds */
    Mix_FreeChunk( gSoundScore );
    Mix_FreeChunk( gSoundBounceWall );
    Mix_FreeChunk( gSoundBouncePaddle );
    gSoundScore = NULL;
    gSoundBounceWall = NULL;
    gSoundBouncePaddle = NULL;

    /*  Free the fonts */
    TTF_CloseFont( gFont );
    TTF_CloseFont( gFontHelp );
    gFont = NULL;
    gFontHelp = NULL;

    /*  Quit SDL and its subsystems */
    Mix_Quit();
    TTF_Quit();
    SDL_Quit();
}



/*
--------------------------------------------------------------------------------
                                HANDLE_COLLISION
--------------------------------------------------------------------------------
*/
void handle_collision( void )
{
    /*  Get the colliders for the paddles.  It's just easier to type this way */
    SDL_Rect p[ 2 ];
    p[ 0 ] = paddles[ 0 ]->get_collider();
    p[ 1 ] = paddles[ 1 ]->get_collider();

    /*  Check for collision with the paddles */
    for( int i = 0; i < 2; ++i )
    {
        /*  If there was a collision */
        if( ball->collision( p[ i ] ) )
        {
            //ball->hit();                  //  !-  Not implemented
            ball->reset_position();         //  Move ball back

            /*  Again, dear user, my laziness costs you precious RAM and CPU */
            int bx = ball->get_pos_x();
            int by = ball->get_pos_y();

            /*  Hit the side */
            if( ( bx > p[ i ].x + p[ i ].w ) ||
                    ( bx + ball->get_width() < p[ i ].x ) )
            {
                /*
                 *  Find the difference (relative Y) between the center of the
                 *  ball and the center of the paddle
                 */
                int relIntY = ( ( by + ( ball->get_width()/2 ) ) -  // b center
                    ( p[ i ].y + ( p[ i ].h/2 ) ));                 // p center

                /*
                 *  Divide the difference by half of the paddle's height to
                 *  normalize it (give it an absolute value between 0 and 1)
                 */
                double normRelIntY = ( relIntY / (p[i].h/2.f) );

                /*
                 *  Multiply the normalized value by the max angle we want in
                 *  order to get the Y angle of the bounce (in degrees)
                 */
                double bounceAngle = ( normRelIntY *
                        ball->get_max_bounce_angle());


                /*
                 *  I don't actually understand this bit yet.  I just found it
                 *  on the internet.  We're setting the Y direction to the
                 *  negative sine of the angle.
                 *
                 *  What I THINK is happening is that M_PI/180.0f converts
                 *  the bounce angle to radians, and then we get the sine of the
                 *  radians for... some reason.  Yeah, trigonometry baffles me.
                 */
                ball->set_dy(
                        ball->get_velocity_y() *
                        std::sin( bounceAngle * M_PI/180.0f ) );

                /*  Reverse X direction */
                ball->flip_dx();
            }

            /*  Hit the top or bottom */
            else
            {
                ball->flip_dy();
            }

            /*  Play sound */
            if( ! ( Mix_Playing( 2 ) > 0 ) && ! mute )
            {
                Mix_PlayChannel( 2, gSoundBouncePaddle, 0 );
            }
        }
    }

}


/*
--------------------------------------------------------------------------------
                                   INIT_SOUND
--------------------------------------------------------------------------------
*/
bool init_sound( void )
{
    /*  Sound effect for when a player or AI scores */
    gSoundScore = Mix_LoadWAV( "data/blip-low.wav" );
    if( gSoundScore == NULL )
    {
        printf("ERROR:  Could not open score sound.  SDL mixer error:  %s\n",
                Mix_GetError() );
        return( false );
    }

    /*  Sound effect for the ball hitting the celiing/floor */
    gSoundBounceWall = Mix_LoadWAV( "data/blip-med.wav" );
    if( gSoundBounceWall == NULL )
    {
        printf("ERROR:  Could not open wall bounce sound.  Mixer error:  %s\n",
                Mix_GetError() );
        return( false );
    }

    /*  Sound effect for the ball hitting a paddle */
    gSoundBouncePaddle = Mix_LoadWAV( "data/blip-hi.wav" );
    if( gSoundBouncePaddle == NULL )
    {
        printf("ERROR:  Can't open paddle bounce sound.  Mixer error:  %s\n",
                Mix_GetError() );
        return( false );
    }

    /*  If it all loaded correctly, awesome */
    return( true );
}



/*
--------------------------------------------------------------------------------
                                   INIT_COLOR
--------------------------------------------------------------------------------
*/
void init_color( SDL_Color *color, int r, int g, int b )
{
    /*  This function exists solely to save a bit of typing */
    color->r = r;
    color->g = g;
    color->b = b;
}



/*
--------------------------------------------------------------------------------
                                  INIT_COLORS
--------------------------------------------------------------------------------
*/
void init_colors( void )
{
    /*  Create all the colors */
    for( int i = 0; i < TOTAL_COLORS; ++i )
    {
        colors[ i ] = new SDL_Color;
    }

    /*  Initialize all the colors */
    init_color( colors[ COLOR_WHITE ], 255, 255, 255 );
    init_color( colors[ COLOR_RED ], 255, 0, 0 );
    init_color( colors[ COLOR_YELLOW ], 255, 255, 0 );
    init_color( colors[ COLOR_PURPLE ], 255, 0, 255 );
    init_color( colors[ COLOR_CYAN ], 0, 255, 255 );
    init_color( colors[ COLOR_BLUE ], 0, 0, 255 );
    init_color( colors[ COLOR_GREEN ], 0, 255, 0 );
    init_color( colors[ COLOR_BLACK ], 0, 0, 0 );

    fgColor = *colors[ COLOR_WHITE ];           //  Assign foreground color
    bgColor = *colors[ COLOR_BLACK ];           //  Assign background color
    outlineColor = *colors[ COLOR_WHITE ];      //  Assign outline color
}



/*
--------------------------------------------------------------------------------
                                  COLORS_MATCH
--------------------------------------------------------------------------------
*/
bool colors_match( SDL_Color c1, SDL_Color c2 )
{
    /*  Check to see if the colors are the same.  If they are, return true. */
    if( c1.r == c2.r && c1.g == c2.g && c1.b == c2.b )
        return( true );

    //  Otherwise
    return( false );
}



/*
--------------------------------------------------------------------------------
                                  CHANGE_COLOR
--------------------------------------------------------------------------------
*/
void change_color( SDL_Color &color )
{
    int currentIndex = 0;

    /*  Go through all of the available colors until we hit the one we're on */
    for( int i = 0; i < TOTAL_COLORS; ++i )
    {
        /*  If the colors match, we've found it */
        if( colors_match( *colors[ i ], color ) )
        {
            currentIndex = i;
            break;
        }
    }

    /*  If the next index would go over the colors available, set it to 0 */
    if( currentIndex + 1 == TOTAL_COLORS )
        currentIndex = 0;
    else
        ++currentIndex;

    /*  Set the color to the color index */
    color = *colors[ currentIndex ];


    /*  See if they need an outline */
    if( colors_match( fgColor, bgColor ) )
    {
        outline = true;

        /* If the background color is black, set the outline to white */
        if( colors_match( bgColor, *colors[ COLOR_BLACK ] ) )
            outlineColor = *colors[ COLOR_WHITE ];

        /*  If it's any other color, set the outline to black */
        else
            outlineColor = *colors[ COLOR_BLACK ];
    }

    /*  If the background and foreground aren't the same, don't outline */
    else
        outline = false;

    /*  Reconfigure help screen text */
    config_help_textures();
}



/*
--------------------------------------------------------------------------------
                                  INIT_MIDLINE
--------------------------------------------------------------------------------
*/
void init_midline( void )
{
    midRect.w = 4;
    midRect.h = 15;

    midRect.x = ( SCREEN_WIDTH - midRect.w ) / 2;
    midRect.y = 0;
}



/*
--------------------------------------------------------------------------------
                                    INIT_OSD
--------------------------------------------------------------------------------
*/
void init_OSD( void )
{
    /*  Create the third element of the gTextures array (OSD texture */
    gTextures[ 2 ] = new Texture();

    /*  Set the OSD object texture index to match the new texture object */
    osd.set_texture_index( 2 );

    /*  Update the OSD */
    osd.update();
}



/*
--------------------------------------------------------------------------------
                                 RENDER_MIDLINE
--------------------------------------------------------------------------------
*/
void render_midline( void )
{
    /*  Top-mose position is 0 (top of screen) */
    midRect.y = 0;

    /*  If we're outline, set draw color to outline color */
    if( outline )
    {
        SDL_SetRenderDrawColor( gRenderer,
                outlineColor.r, outlineColor.g, outlineColor.b, 255 );
    }
    else
    {
        SDL_SetRenderDrawColor( gRenderer,
                    fgColor.r, fgColor.g, fgColor.b, 255 );
    }


    /*  We draw a dashed midline down the center of the screen */
    for( int i = 0; i < midRect.h * 2; ++i )
    {

        if( outline )
            SDL_RenderDrawRect( gRenderer, &midRect );
        else
            SDL_RenderFillRect( gRenderer, &midRect );

        midRect.y += midRect.h * 3;
    }
}


/*
================================================================================

                                      MAIN

================================================================================
*/
int main( int argc, char *argv[] )
{
    /*
     *  Basic argument handling
     */
    if( argc == 2 )
    {
        /*  If the user wants help */
        if( ( strncmp( argv[1], "-h", strlen("-h") ) == 0 ) ||
                ( strncmp( argv[1], "--help", strlen("--help")) == 0 ) )
        {
            print_help();
            return(0);
        }

        /*  If the user wants version/author info */
        else if( strncmp( argv[1], "--version", strlen("--version") ) == 0 )
        {
            print_version();
            return(0);
        }

        /*  Anything else */
        else
        {
            printf("ERROR:  Invalid argument:  %s\n", argv[1] );
            return(1);
        }
    }

    /*  If we couldn't init, bail out */
    if( ! init() )
    {
        printf("ERROR:  Could not init\n");
        return(1);
    }

    /*  If we couldn't load the fonts, abandon ship */
    if( ! init_fonts() )
    {
        printf("ERROR:  Could not init font\n");
        return(1);
    }

    /*  If we couldn't load the sounds, uh... return 1 */
    if( ! init_sound() )
    {
        printf("ERROR:  Could not init sound\n");
        return(1);
    }

    /*  'Grab' the mouse cursor in the window, and hide it as well */
    SDL_SetRelativeMouseMode( SDL_TRUE );

    /*  Init all the things */
    init_colors();
    init_paddles();
    init_ball();
    init_textures();
    init_help_textures();
    init_game_over_textures();
    init_scores();
    init_OSD();
    init_midline();

    /*  Event queue */
    SDL_Event e;


    /*  While the user hasn't quit, do everything */
    while( quit == false )
    {
        /*  If the sound is off, kill all sounds */
        if( mute )
            Mix_HaltChannel( -1 );

        /*  Handle all events */
        events_main( e );

        /*  If we're resetting the ball */
        if( reset )
        {
            /*  Stick the ball in the middle of the screen */
            ball->set_position(
                    ( SCREEN_WIDTH - ball->get_width() ) / 2,
                    ( SCREEN_HEIGHT - ball->get_height( )) / 2  );

            /*  Reverse whichever direction the ball was going */
            if( ball->get_dx() > 0 )
                ball->set_dx( -( ball->get_dx()) );
            else
                ball->set_dx( ball->get_dx() );

            /*  Give it zero vertical direction */
            ball->set_dy( 0 );

            /*  Reset left paddle position */
            paddles[ 0 ]->set_position(
                    paddles[ 0 ]->get_pos_x(),
                    ( SCREEN_HEIGHT - paddles[ 0 ]->get_height() ) / 2 );

            /*  Reset right paddle position */
            paddles[ 1 ]->set_position(
                    paddles[ 1 ]->get_pos_x(),
                    ( SCREEN_HEIGHT - paddles[ 1 ]->get_height() ) / 2 );

            /*  Wait a second before allowing the player to launch the ball */
            if( pausedTicks <= 60 )
                ++pausedTicks;
            else
                reset = false;
        }

        /*  If the game isn't paused, continue as normal */
        else if( ! paused )
        {
            /*  Handle the movement for the paddle */
            paddles[ 0 ]->move();
            paddles[ 1 ]->move();

            /*  Move the ball */
            ball->move();

            /*  Send the joystick values to the joysticks */
            if( SDL_NumJoysticks() == 1 )
                paddles[ 0 ]->move_player_joystick( valuePad1 );
            else if( SDL_NumJoysticks() == 2 )
            {
                paddles[ 0 ]->move_player_joystick( valuePad1 );
                paddles[ 1 ]->move_player_joystick( valuePad2 );
            }
        }

        /*  Render everything */
        render_main();
    }

    /*  Close out the program */
    close();

    return(0);
}
