#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
Texture::Texture( void )
{
    /*  Null the texture pointer */
    mTexture = NULL;

    /*  Init position */
    mPos.x = 0;
    mPos.y = 0;

    /*  Init dimensions */
    mWidth = 0;
    mHeight = 0;
}


/*
--------------------------------------------------------------------------------
                                   DESTRUCTOR
--------------------------------------------------------------------------------
*/
Texture::~Texture( void )
{
    /*  Free the texture */
    free_texture();
}


/*
--------------------------------------------------------------------------------
                             CREATE / FREE TEXTURE
--------------------------------------------------------------------------------
*/

bool Texture::create_texture_from_string( TTF_Font *font, const char *string,
        SDL_Color textColor )
{
    /*  Free the texture if it exists */
    free_texture();

    /*  Create an SDL surface from rendered text */
    SDL_Surface *tempSurface = TTF_RenderText_Blended( font, string, textColor);
    if( tempSurface == NULL )
    {
        printf("ERROR:  Cannot create surface from string.  TTF Error:  %s\n",
                TTF_GetError() );
        return( false );
    }

    /*  Create a texture from the SDL surface */
    mTexture = SDL_CreateTextureFromSurface( gRenderer, tempSurface );
    if( mTexture == NULL )
    {
        printf("ERROR:  Cannot create texture from surface.  SDL Error:  %s\n",
                SDL_GetError() );
        return( false );
    }

    /*  Get the dimensions from the surface */
    mWidth = tempSurface->w;
    mHeight = tempSurface->h;

    /*  Free the surface */
    SDL_FreeSurface( tempSurface );

    /*  If we're this far, we're good */
    return( true );
}


void Texture::free_texture( void )
{
    /*  If the texture exists, kill it */
    if( mTexture != NULL )
    {
        SDL_DestroyTexture( mTexture );
        mTexture = NULL;
        mWidth = 0;
        mHeight = 0;
    }
}


/*
--------------------------------------------------------------------------------
                                 COLOR / ALPHA
--------------------------------------------------------------------------------
*/

/*  Modulate color */
void Texture::set_color( Uint8 red, Uint8 green, Uint8 blue )
{
    SDL_SetTextureColorMod( mTexture, red, green, blue );
}


/*  Modulate alpha */
void Texture::set_alpha( Uint8 alpha )
{
    SDL_SetTextureAlphaMod( mTexture, alpha );
}


/*  Get alpha value */
void Texture::get_alpha( Uint8 *alpha )
{
    SDL_GetTextureAlphaMod( mTexture, alpha );
}



/*
--------------------------------------------------------------------------------
                                     RENDER
--------------------------------------------------------------------------------
*/
void Texture::render( int x, int y )
{
    /*  If we're given valid coordinates, render to those */
    if( x > 0 )
        mPos.x = x;
    if( y > 0 )
        mPos.y = y;

    /*  Destination rect */
    SDL_Rect destR = { mPos.x, mPos.y, mWidth, mHeight };

    /*  Copy to the rendering context */
    SDL_RenderCopy( gRenderer, mTexture, NULL, &destR );
}


/*
--------------------------------------------------------------------------------
                             POSITION / DIMENSIONS
--------------------------------------------------------------------------------
*/

void Texture::set_position( int x, int y )      //  Set X and Y positions
{
    mPos.x = x;
    mPos.y = y;
}

int Texture::get_width( void )                  //  Get width
{
    return( mWidth );
}


int Texture::get_height( void )                 //  Get height
{
    return( mHeight );
}
